# rethreader.py
# Scours a Twitter JSON backup for threads
# Public domain.
# Revision 3 - nicer formatting for output

## First step:  turn tweets.js into a valid JSON file.

import json, os, sys, datetime

if os.path.exists('tweets.js') is False and os.path.exists('tweets.json') is False:
    print("Please copy the tweets.js from the /data folder in your Twitter directory to the same path as backup-rethreader.py, and restart the program.")
    sys.exit(1)

if os.path.exists('tweets.json') is False:
    print("Converting tweets.js to .json (and fixing a potential nasty bug in the Twitter exporter along the way)...\n")
    f = open('tweets.js', encoding="utf-8")
    text = f.read()
    f.close()

    firstlineends = text.find("\n")
    text = "[\n" + text[firstlineends+1:]

    ## Fixes multiline error in some Twitter exports:  \n as both literal and code
    if "\n\\n" in text:
        print("Error string found.  Fixing...\n")
        text = text.replace("\n\\n", "\\n")

    f = open('tweets.json', 'w', encoding='utf8')
    f.write(text)
    f.close()

## Get data from JSON
f = open('tweets.json', encoding='utf8')
data = json.load(f)
print("Loaded json...\n")
f.close()

tweets = {} # Given an id, return its text.
times = {} # Given an id, return its time.
replies = {} # Given an id, return its replies.  One to many.
parents = {} # Given an id, return its parent.  One to one.
roots = [] # Only lists roots.

# Load tweets with texts of tweets.  Must be completely done before next steps.
for i in data:
    tweet = i['tweet']
    tweetid = tweet['id']
    if 'created_at' in tweet:
        times[tweetid] = tweet['created_at'].strip()
    if 'full_text' in tweet:
        tweets[tweetid] = tweet['full_text'].strip()

# Load indexes of parent tweets and replies.
for i in data:
    tweet = i['tweet']
    if 'in_reply_to_status_id' in tweet:
        tweetid = tweet['id']
        parentid = tweet['in_reply_to_status_id']
        
        if parentid in tweets:
            # store the parents of this tweet (for lookup)
            parents[tweetid] = parentid

            # and add this tweet to the replies for that tweet (for printing)
            if parentid not in replies:
                replies[parentid] = []
            replies[parentid].append(tweetid)

# Recurse back to ultimate parent - or use this parent, if needed.
def getroot(tweetid):
    if tweetid in parents:
        return getroot(parents[tweetid])
    else:
        return(tweetid)

# Load index of roots.
for tweetid, _ in replies.items():
    root = getroot(tweetid)
    if root not in roots:
        roots.append(root)

## Finally, output.
output = open('threads.txt', 'w', encoding='utf8')

# Print out one tweet and its replies, indenting next pass further if multiple replies in this thread...
def printtweet(tweetid, depth=0):
    if tweetid in tweets:
        output.write(" "*depth + tweets[tweetid] + "\n\n")
    if tweetid in replies:
        if len(replies[tweetid])==1:
            printtweet(replies[tweetid][0],depth)
        else:
            for t in replies[tweetid]:
                printtweet(t, depth+1)

# Now print them out.
for root in roots:
    if root in times:
        output.write(f"Thread beginning {times[root]}:\n\n")
    printtweet(root)
    output.write("\n###\n\n")

output.close()


print("Output in threads.txt")
